@@ -25,11 +25,12 @@ def decimal_change_base(number, base=16, characters="0123456789abcdefghijklmnopq
 
 
 def get_float_part(value, precision=7):
-    value = abs(value - int(value))
-    value *= 10 ** precision
-    value = int(value)
-    return value
-
+    value = abs(value)
+    mod = 10 ** precision
+    value_all = int(value * mod)
+    value_int = int(value) * mod
+    value_float = value_all - value_int
+    return value_float
 
 def float_split(value, precision=7):
     if value >= 0:
